﻿using System;
using System.Xml.Serialization;
using System.IO;
using System.Xml;
using System.Text;
using System.Reflection;

namespace TriTech.Common
{
    public class XmlSerialize
    {
        /// <summary>
        /// serialize the object into an XML-format string.
        /// </summary>
        /// <param name="oObject">The object to be converted</param>
        /// <param name="objectType">Serialize the object using the given type (the object type must be of this type or inherit from it)</param>
        /// <param name="indentXml">True - if the xml should be indented.</param>
        /// <returns>XML String</returns>
        public static string ConvertClassToXMLString(object oObject, Type objectType, bool indentXml)
        {
            // Create an instance of the XmlSerializer class;
            // specify the type of object to serialize.
            XmlSerializer serializer = new XmlSerializer(objectType);
            StringWriter stringWriter = new StringWriter();

            // Serialize the object, and return the string.
            if (indentXml)
            {
                serializer.Serialize(stringWriter, oObject);
            }
            else
            {
                //// using a XmlWriter will make the xml unindented
                //XmlWriterSettings settings = new XmlWriterSettings();
                //settings.Indent = false;
                //settings.OmitXmlDeclaration = true;
                ////settings.ConformanceLevel = ConformanceLevel.Fragment;
                //XmlWriter xmlWriter = XmlWriter.Create(stringWriter, settings);
                //serializer.Serialize(xmlWriter, oObject);
                // using a XmlTextWrite will make the xml unindented
                XmlTextWriter textWriter = new XmlTextWriter(stringWriter);
                textWriter.Formatting = Formatting.None;
                serializer.Serialize(textWriter, oObject);
            }

            return stringWriter.ToString();
        }

        /// <summary>
        /// serialize the object into an XML-format string, using the object's type.
        /// </summary>
        /// <param name="oObject">The object to be converted</param>
        /// <param name="indentXml">True - if the xml should be indented.</param>
        /// <returns>XML String</returns>
        public static string ConvertClassToXMLString(object oObject, bool indentXml)
        {
            return ConvertClassToXMLString(oObject, oObject.GetType(), indentXml);
        }

        /// <summary>
        /// Serialize the object into an XML-format string, using the object's type.
        /// This overload support encoding.
        /// </summary>
        /// <param name="oObject">Object instance to be serialized.</param>
        /// <param name="encoding">Encoding to be used.</param>
        /// <returns></returns>
        public static string ConvertClassToXMLString(object oObject, Encoding encoding)
        {
            return ConvertClassToXMLString(oObject, encoding, null);
        }

        /// <summary>
        /// Serialize the object into an XML-format string, using the object's type.
        /// This overload support encoding and prefixes for namespaces.
        /// </summary>
        /// <param name="oObject">Object instance to be serialized.</param>
        /// <param name="encoding">Encoding to be used.</param>
        /// <param name="namespaces">XmlSerializerNamespaces to be used to decorate the XML message with defined namespace prefixes.</param>
        /// <returns>XML string.</returns>
        public static string ConvertClassToXMLString(object oObject, Encoding encoding, XmlSerializerNamespaces namespaces)
        {
            XmlSerializer serializer = new XmlSerializer(oObject.GetType());
            using (MemoryStream memoryStream = new MemoryStream())
            {
                XmlWriterSettings settings = new XmlWriterSettings()
                {
                    Encoding = encoding
                };

                using (XmlWriter writer = XmlWriter.Create(memoryStream, settings))
                {
                    if (namespaces != null && namespaces.Count > 0)
                    {
                        serializer.Serialize(writer, oObject, namespaces);
                    }
                    else
                    {
                        serializer.Serialize(writer, oObject);
                    }
                }

                return encoding.GetString(memoryStream.ToArray());
            }
        }

        /// <summary>
        /// serialize the object into an XML-format file.
        /// </summary>
        /// <param name="oObject">The object to be converted</param>
        /// <param name="indentXml">True - if the xml should be indented.</param>
        /// <param name="xmlFileName">The xml file name (and path).</param>
        public static void ConvertClassToXMLFile(object oObject, bool indentXml, string xmlFileName)
        {
            // Create an instance of the XmlSerializer class;
            // specify the type of object to serialize.
            XmlSerializer serializer = new XmlSerializer(oObject.GetType());
            StreamWriter streamWriter = new StreamWriter(xmlFileName);

            // Serialize the object, and return the string.
            if (indentXml)
            {
                serializer.Serialize(streamWriter, oObject);
            }
            else
            {
                // using a XmlTextWrite will make the xml unindented
                XmlTextWriter textWriter = new XmlTextWriter(streamWriter);
                textWriter.Formatting = Formatting.None;
                serializer.Serialize(textWriter, oObject);
            }

            streamWriter.Close();
        }

        /// <summary>
        /// Deserialize the XML string into an object.
        ///		Call this function like this:
        ///		messageType msg = (messageType)ms.ConvertXMLtoClass(xmlString, typeof(messageType));
        /// </summary>
        /// <param name="sXml">XML String</param>
        /// <param name="msgType">The object type</param>
        /// <param name="ignoreErrors">If true, then no exception thrown when unknown nodes/attributes found. They just ignored.</param>
        /// <returns>The deserialized object</returns>
        public static Object ConvertXMLtoClass(string sXml, Type msgType, bool ignoreErrors)
        {
            XmlSerializer serializer = new XmlSerializer(msgType);

            if (ignoreErrors)
            {
                serializer.UnknownNode += new XmlNodeEventHandler(serializer_UnknownNodeIgnored);
                serializer.UnknownAttribute += new XmlAttributeEventHandler(serializer_UnknownAttributeIgnored);
            }
            else
            {
                serializer.UnknownNode += new XmlNodeEventHandler(serializer_UnknownNode);
                serializer.UnknownAttribute += new XmlAttributeEventHandler(serializer_UnknownAttribute);
            }

            return serializer.Deserialize(new XmlTextReader(new StringReader(sXml)));
        }

        /// <summary>
        /// Deserialize the XML string into an object.
        ///		Call this function like this:
        ///		TriggerRule triggerRule = Deserialize<TriggerRule>(xml);
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="xml"></param>
        /// <returns></returns>
        public static T Deserialize<T>(string xml)
        {
            return (T)ConvertXMLtoClass(xml, typeof(T), true);
        }


        /// <summary>
        /// Deserialize the XML file into a class 
        ///		Call this function like this:
        ///		messageType msg = (messageType)ms.ConvertXmlFileToClass(xmlFileName, typeof(messageType));
        /// </summary>
        /// <param name="sXmlFileName">The file name</param>
        /// <param name="oType">The object type</param>
        /// <returns>The deserialized object</returns>
        public static object ConvertXmlFileToClass(string sXmlFileName, Type oType)
        {
            // return null, if file doesn't exists
            if (!File.Exists(sXmlFileName)) return null;

            // Create an instance of the XmlSerializer class;
            // specify the type of object to be deserialized.
            XmlSerializer serializer = new XmlSerializer(oType);
            // If the XML document has been altered with unknown 
            //	nodes or attributes, handle them with the 
            //	UnknownNode and UnknownAttribute events.
            serializer.UnknownNode += new
                XmlNodeEventHandler(serializer_UnknownNode);
            serializer.UnknownAttribute += new
                XmlAttributeEventHandler(serializer_UnknownAttribute);

            // A FileStream is needed to read the XML document.
            StreamReader reader = new StreamReader(sXmlFileName);

            // Use the Deserialize method to restore the object's state with
            //	data from the XML document. 
            object result = serializer.Deserialize(reader);
            reader.Close();
            return result;
        }

        private static void serializer_UnknownNode(object sender, XmlNodeEventArgs e)
        {
            string sError = "Invalid XML - Unknown Node:" + e.Name + "\t" + e.Text;
            throw new Exception(sError);
        }

        private static void serializer_UnknownAttribute(object sender, XmlAttributeEventArgs e)
        {
            System.Xml.XmlAttribute attr = e.Attr;
            string sError = "Invalid XML - Unknown attribute " + attr.Name + "='" + attr.Value + "'";
            throw new Exception(sError);
        }

        private static void serializer_UnknownNodeIgnored(object sender, XmlNodeEventArgs e)
        {
        }

        private static void serializer_UnknownAttributeIgnored(object sender, XmlAttributeEventArgs e)
        {
        }

        /// <summary>
        /// Returns the xml value of an enum item generated by xsd.exe
        /// </summary>
        /// <param name="enumItem">The enum item</param>
        /// <returns>the XML String representaion of the item</returns>
        public static string GetEnumItemXmlName(Enum enumItem)
        {
            try
            {
                FieldInfo fieldInfo = enumItem.GetType().GetField(enumItem.ToString());
                object[] attributes = fieldInfo.GetCustomAttributes(typeof(XmlEnumAttribute), true);
                XmlEnumAttribute attribute = (XmlEnumAttribute)attributes[0];
                return attribute.Name;
            }
            catch { }
            return null;
        }

    }
}
